/**The MIT License (MIT)
Copyright (c) 2019 by ArtSuzhou 
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/


#include <Wire.h>  // Comes with Arduino IDE
#include <LiquidCrystal_I2C.h>  //https://github.com/fdebrabander/Arduino-LiquidCrystal-I2C-library
#include <RunningAverage.h> //https://github.com/RobTillaart/Arduino/blob/master/libraries/RunningAverage/RunningAverage.h

RunningAverage myRA(50);

int samples = 0;
int alarm_state = 0; // if Alarm function is enabled/disabled
int isAlarm = 0; // if the alarm should sound
int isLow = 0;  // if low air flow has been detected
int sensorPin = A0;   

float Vout=0;
float P=0;
float PH2O=0;
float Diff=0;

int offset = 0;
int offset_size =10;
int p_mean_size = 5;

#define NUMALARM 5  // number of buzzer sound pattern
#define CUTOFF 100  // cut off value used to determine alarm state

const int buzz_pin = 5; //pin for buzzer
const int buttonPin = 6; //pin for button
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled
unsigned long debounceDelay = 50;    // the debounce time; increase if the output flickers
unsigned long lcdDelay = 30000; //the lcd delay time 30 seconds
unsigned long lastAlarmTime = 0 ; //the last time the low flow was detected
unsigned long alarmDelay = 10000; //the alarm dealy time 10 seconds

int buttonState;
int lastButtonState = HIGH;

// initialize the library with the numbers of the interface pins. SDA-->A4, SCL-->A5
LiquidCrystal_I2C lcd(0x27, 16, 2);  // set the LCD address to 0x27 for a 16 chars and 2 line display

void setup() {

  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(buzz_pin,OUTPUT);  // make buzz_pin an output for buzzer

  Serial.begin(115200); 
  Serial.println("init...");
  myRA.clear(); // explicitly start clean
   
  // Turn on the blacklight and print a message.
  lcd.begin();
  lcd.backlight();
  introScreen();
  lcd.clear(); // clear the lcd
  
  // setup and calculate offset
  for (int ii=0;ii<offset_size;ii++){
    offset += analogRead(sensorPin)-(1023/2);
  }
  offset /= offset_size;
   
  Serial.print("Offset = ");
  Serial.print(offset);
  Serial.print("\t");
  Serial.println("Sensor Ok");

  lcd.setCursor(0,0);
  lcd.print("Offset = ");
  lcd.print(offset);
  delay(1000);
  
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("CPAP Monitor");  
  lcd.setCursor(0,1);
  lcd.print("is Ready"); 
  delay(2000);
  lcd.clear();
}

void loop() {
  int reading = digitalRead(buttonPin);
  
  if (reading != lastButtonState) {
    Serial.println("Button changed");  
    lastDebounceTime = millis();
  }

 if ((millis() - lastDebounceTime) > debounceDelay) {
    if (reading != buttonState) {
      buttonState = reading;
      if (buttonState == LOW) {
         alarm_state = !alarm_state;
      }
    }
  }

  lastButtonState = reading;
  
  float sensorValue = 0;
 
  // average a few ADC readings for stability
  for (int ii=0;ii<p_mean_size;ii++){
    sensorValue += analogRead(sensorPin)-offset;
  }
    sensorValue /= p_mean_size;
    
  // remove noise near zero
  if (abs(sensorValue-512.0)<3.0){
    sensorValue = 512.0;    
  }
    
  // sensorValue = analogRead(sensorPin)-offset; 
   Vout=(5*sensorValue)/1024.0;
   P=(Vout-2.5)*1000;
   PH2O=(Vout-2.5)*1000*0.10197162129779*0.10;   //convert Pa to CmH20
       
  myRA.addValue(P);
  samples ++;
  
  if (alarm_state ==1 )
  {
    lcd.setCursor(15,1);
    lcd.print("*");
    if(((millis() - lastDebounceTime) < lcdDelay) or isLow == 1 ){
        lcd.backlight(); // LCD backlight delayed off, and turn on on alarm 
      }
    else{
        lcd.noBacklight(); // turn off LCD backlight
      }
    }
  else
  {
    if(((millis() - lastDebounceTime) < lcdDelay) or Diff >0.0 )
    {
    lcd.backlight();// LCD backlight keeps on if alarm is not enabled
    lcd.setCursor(15,1);
    lcd.print("-");
    }
    else
    {
     lcd.noBacklight(); //Turn off LCD backlight if the results are zero
     lcd.setCursor(15,1);
     lcd.print("-");
    }
  }

  if (samples % 100 == 0)
  {
  displayP();
  }      
       
  if (samples == 300)
  {
  Diff = myRA.getMax()-myRA.getMin();

  if (Diff <= CUTOFF and isLow ==0 ){
  lastAlarmTime = millis();
  isLow = 1;
  }
  else{
  if (Diff > CUTOFF)
  {
  lastAlarmTime = millis();
  isLow = 0;
  }
  }
  
  if (isLow ==1 and ((millis() - lastAlarmTime) > alarmDelay)){
  isAlarm = 1;  
  }
  else{
  isAlarm = 0;  
  }  
    

  displayRA();

  if (alarm_state ==1 and isAlarm ==1){
  lcd.backlight();
  lcd.setCursor(0,1);
  lcd.print("Check Mask!");
  alarm();  
  }
 
  samples = 0;
  myRA.clear();
  }
      
  delay(10);   
    
}

  void displayP()
{
  char poutstr[8];
  dtostrf(P,8,2, poutstr);
  char houtstr[6];
  dtostrf(PH2O,6,2, houtstr);
  
  lcd.setCursor(0,0);
  lcd.print(poutstr);
  lcd.setCursor(8, 0);
  lcd.print("P");
  
  lcd.setCursor(9,0);
  lcd.print(houtstr);
  lcd.setCursor(15, 0);
  lcd.print("H");
  }

  void displayRA()
{
  char minoutstr[4];
  dtostrf(myRA.getMin(),4,0, minoutstr);
  char maxoutstr[4];
  dtostrf(myRA.getMax(),4,0, maxoutstr);
  char diffoutstr[4];
  dtostrf(Diff,4,0, diffoutstr);
  lcd.setCursor(0,1);
  lcd.print("Min.  Max.  Diff.");
  delay(1000);   
  lcd.setCursor(0,1);
  lcd.print("                ");
  lcd.setCursor(0,1);  
  lcd.print(minoutstr);
  lcd.setCursor(5, 1);
  lcd.print(maxoutstr);
  lcd.setCursor(10, 1);
  lcd.print(diffoutstr);  
}  

void introScreen()
{
  lcd.setCursor(0, 0);
  lcd.print("CPAP Monitor");
  lcd.setCursor(0, 1);
  lcd.print("Initializing...");
  delay(2000);
  lcd.clear();
}

void alarm()
{
   uint8_t i;
   for (i=0; i< NUMALARM; i++) {
   digitalWrite(buzz_pin, HIGH);
   delay(500);
   digitalWrite(buzz_pin, LOW); 
   delay(500);
   }
}
